package com.digiwin.athena.app.ptc.service.receivables;

import com.alibaba.fastjson.TypeReference;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.common.enums.TabStatusEnums;
import com.digiwin.athena.app.ptc.infra.entity.CollectionDetailEntity;
import com.digiwin.athena.app.ptc.infra.repository.CollectionDetailRepository;
import com.digiwin.athena.app.ptc.infra.repository.ReceivablesDetailRepository;
import com.digiwin.athena.app.ptc.infra.service.CollectionDetailService;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import com.digiwin.athena.opt.common.util.DateUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * @auther: zhenggl
 * @date: 2023/9/12
 */
@Log4j2
@Service
public class ReceivablesInitialCreateEAIService extends AbsEAIService {

    @Resource
    private CollectionDetailRepository collectionDetailRepository;

    @Resource
    private ReceivablesDetailRepository receivablesDetailRepository;

    @Resource
    private CollectionDetailService collectionDetailService;


    @Override
    public String getServiceName() {
        return ReceivablesUtil.RECEIVABLES_INITIAL_CREATE;
    }


    //随机赋值list
    private List<CollectionDetailEntity> list = Arrays.asList(
            new CollectionDetailEntity("鼎捷A半导体有限公司","dj001","张三","186****9498"),
            new CollectionDetailEntity("鼎捷B半导体有限公司","dj002","李四","158****4794"),
            new CollectionDetailEntity("鼎捷C半导体有限公司","dj003","王五","150****9529"),
            new CollectionDetailEntity("鼎新A半导体有限公司","dj004","赵六","139****0530"),
            new CollectionDetailEntity("鼎新B半导体有限公司","dj005","陈七","136****5333"),
            new CollectionDetailEntity("鼎新C半导体有限公司","dj006","刘八","188****9299"),
            new CollectionDetailEntity("鼎华A半导体有限公司","dj007","韩九","130****3555"),
            new CollectionDetailEntity("鼎华B半导体有限公司","dj008","管十一","138****9999"),
            new CollectionDetailEntity("鼎华C半导体有限公司","dj009","谢十二","187****6160"));


    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {


        Long tenantSid = SecurityUtil.getUserProfile().getTenantSid();

        //取出已完成的数据
        LambdaQueryWrapper<CollectionDetailEntity> lmq = new LambdaQueryWrapper<>();
        lmq.eq(CollectionDetailEntity::getTenantSid,tenantSid);
        lmq.eq(CollectionDetailEntity::getStatus, TabStatusEnums.COMPLETED.getValue());
        List<CollectionDetailEntity> list = collectionDetailService.list(lmq);

        //删除应收数据
        if (CollectionUtils.isNotEmpty(list)){
            receivablesDetailRepository.deleteBatch(list,tenantSid);
        }


        Random random = new Random();
        //随机新增
        for (CollectionDetailEntity collectionDetailEntity : this.list) {
            int randomNumber = random.nextInt(100);
            int nextInt = random.nextInt(30);
            int dayInt = random.nextInt(10);
            collectionDetailEntity.setReceivableNo("RE-"+DateUtils.currentTimeString());
            collectionDetailEntity.setReceivableNo("SEQ-"+DateUtils.currentTimeString());
            collectionDetailEntity.setStatus("1");
            collectionDetailEntity.setSalesOrder("SO-"+DateUtils.currentTimeString());
            collectionDetailEntity.setSalesOrderNumber("SON-"+DateUtils.currentTimeString());
            collectionDetailEntity.setContractNo(UUID.randomUUID().toString());
            collectionDetailEntity.setPriceTax(new BigDecimal(String.valueOf(randomNumber)));
            collectionDetailEntity.setQuantity(new BigDecimal(String.valueOf(nextInt)));
            collectionDetailEntity.setReceivableDate(org.apache.commons.lang.time.DateUtils.addDays(new Date(),dayInt));
            collectionDetailEntity.setSalesmanAssistant("qcsupplierqcuser001");
            collectionDetailEntity.setSalesmanBoss("Sp0001");
            collectionDetailEntity.setSalesman("qcuser001");
            collectionDetailEntity.setProduceBoss("Sp0002");
            collectionDetailEntity.setWorkNo("work-"+DateUtils.currentTimeString());
            collectionDetailEntity.setOverdueDays(String.valueOf(DateUtils.getDayInterval(new Date(),collectionDetailEntity.getReceivableDate())));
        }

        collectionDetailService.saveBatch(this.list);



        return buildOK();
    }
}
